﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;
using DocumentFormat.OpenXml.Packaging;
using System.Linq;
using System.Drawing.Imaging;

namespace OpenXmlPowerTools
{
    public static class ImageElement
    {
        // Bitmap.Width specified in units, where 1 unit = (1/96) inches
        // 1 inch = 914400 emu
        // 1 cm   = 360000 emu
            
        public static int emuPerUnit = 9525;

        public static int[] GetDim(Bitmap imageToInsert)
        {      
            int imageWidth = imageToInsert.Width * emuPerUnit;
            int imageHeight = imageToInsert.Height * emuPerUnit;

            int imageWidthExtent = imageToInsert.Width * emuPerUnit;
            int imageHeightExtent = imageToInsert.Height * emuPerUnit;

            int[] imgDim = new int[] { imageWidth, imageHeight, imageWidthExtent, imageHeightExtent };
            return imgDim;
        }
        
        public static XElement CreateMarkup(Bitmap imageToInsert, string imageRId, int[] imageDim)
        {
            int imageWidth  = imageDim[0];
            int imageHeight = imageDim[1];

            int imageWidthExtent  = imageDim[2];
            int imageHeightExtent = imageDim[3];

            string name = "Image1";

            XElement imageElement =
                new XElement(W.drawing,
                    new XElement(WP.inline,
                        new XAttribute("distT", "0"),
                        new XAttribute("distB", "0"),
                        new XAttribute("distL", "0"),
                        new XAttribute("distR", "0"),
                        //new XAttribute((WP14.wp14 + "anchorId"), "6C10AB47"),  
                        //new XAttribute(WP14.editId, "5687B1E9"),
                        new XElement(WP.extent,
                            new XAttribute("cx", imageWidthExtent),
                            new XAttribute("cy", imageHeightExtent)
                        ),
                        new XElement(WP.effectExtent,
                            new XAttribute("l", "0"),
                            new XAttribute("t", "0"),
                            new XAttribute("r", "0"),
                            new XAttribute("b", "0")
                        ),
                        new XElement(WP.docPr,
                            new XAttribute("id", "1"),   
                            new XAttribute("name", name)                                        
                        ),
                        new XElement(WP.cNvGraphicFramePr,
                            new XElement(A.graphicFrameLocks,
                                new XAttribute(XNamespace.Xmlns + "a", A.a),
                                new XAttribute("noChangeAspect", "1"))
                        ),
                        new XElement(A.graphic,
                                new XAttribute(XNamespace.Xmlns + "a", A.a),
                                new XElement(A.graphicData,
                                    new XAttribute("uri", Pic.pic),
                                    new XElement(Pic._pic,
                                        new XAttribute(XNamespace.Xmlns + "pic", Pic.pic),
                                        new XElement(Pic.nvPicPr,
                                            new XElement(Pic.cNvPr,
                                                new XAttribute("id", "0"),
                                                new XAttribute("name", "")
                                            ),
                                            new XElement(Pic.cNvPicPr)
                                        ),
                                        new XElement(Pic.blipFill,
                                            new XElement(A.blip,
                                                new XAttribute(R.r + "embed", imageRId)
                                            ),
                                            new XElement(A.stretch,
                                                new XElement(A.fillRect)
                                            )
                                        ),
                                        new XElement(Pic.spPr,
                                            new XElement(A.xfrm,
                                                new XElement(A.off,
                                                    new XAttribute("x", "0"),
                                                    new XAttribute("y", "0")
                                                ),
                                                new XElement(A.ext,
                                                    new XAttribute("cx", imageWidth),
                                                    new XAttribute("cy", imageHeight)
                                                )
                                            ),
                                        new XElement(A.prstGeom,
                                            new XAttribute("prst", "rect"),
                                            new XElement(A.avLst)
                                        )
                                    )
                                )
                            )
                        )
                    )
            );
            return imageElement;
        }

        public static ImagePartType GetImagePartType(ImageFormat format)
        {
            if (format.Equals(ImageFormat.Jpeg))
                return ImagePartType.Jpeg;
            else if (format.Equals(ImageFormat.Emf))
                return ImagePartType.Emf;
            else if (format.Equals(ImageFormat.Gif))
                return ImagePartType.Gif;
            else if (format.Equals(ImageFormat.Icon))
                return ImagePartType.Icon;
            else if (format.Equals(ImageFormat.Png))
                return ImagePartType.Png;
            else if (format.Equals(ImageFormat.Tiff))
                return ImagePartType.Tiff;
            else if (format.Equals(ImageFormat.Wmf))
                return ImagePartType.Wmf;
            else
                return ImagePartType.Bmp;
        }


    }

}
