﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using DocumentFormat.OpenXml.Packaging;
using OpenXmlPowerTools;

namespace ReplaceBookmark
{
    class ReplaceBookmarkTest
    {
        static void RemoveDirectory(DirectoryInfo destDirectoryInfo)
        {
            foreach (var dir in destDirectoryInfo.GetDirectories())
            {
                foreach (var file in destDirectoryInfo.GetFiles())
                {
                    file.Delete();
                }
                RemoveDirectory(dir);
            }
            foreach (var file in destDirectoryInfo.GetFiles())
            {
                file.Delete();
            }
            destDirectoryInfo.Delete();
        }

        static void Main(string[] args)
        {
            string sourceDirectory = "../../TestDocumentsSource";
            string destDirectory = "../../TestDocumentsDest";

            DirectoryInfo sdi = new DirectoryInfo(sourceDirectory);
            DirectoryInfo ddi = new DirectoryInfo(destDirectory);

            if (ddi.Exists)
                RemoveDirectory(ddi);
            System.Threading.Thread.Sleep(500);
            ddi.Create();

            Bitmap image, image1, image2, image3, image4, image5;
            bool keepBookmark;
            
            //Test 01: replace text in bookmark A with "abc"
            //         test case-insensitivity for bookmark (using lowercase bookmark)
            File.Copy(sourceDirectory + "/" + "Test01.docx", destDirectory + "/" + "Test01out.docx");
            keepBookmark = true;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test01out.docx", true))
            {
                Console.WriteLine("Test01: replacing text of bookmark A with abc");
                BookmarkReplacer.ReplaceBookmark(doc, "a", "abc", keepBookmark);
            }

            //Test 02: replace bookmark A with image Image01.png
            File.Copy(sourceDirectory + "/Test02.docx", destDirectory + "/" + "Test02out.docx");
            image = new Bitmap(@"../../TestDocumentsSource/Image01.png");
            keepBookmark = false;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test02out.docx", true))
            {
                Console.WriteLine("Test02: replacing bookmark A with Image01.png:");
                BookmarkReplacer.ReplaceBookmark(doc, "A", image, ImageFormat.Png, keepBookmark);
            }

            //Test 03: replace bookmarks b1 and b2 with image Image01.png
            File.Copy(sourceDirectory + "/Test03.docx", destDirectory + "/" + "Test03out.docx");
            image = new Bitmap(@"../../TestDocumentsSource/Image01.png");
            keepBookmark = false;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test03out.docx", true))
            {
                Console.WriteLine("Test03: replacing bookmarks b1 and b2 with Image01.png");
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image, ImageFormat.Png, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b2", image, ImageFormat.Png, keepBookmark);
            }

            //Test 04: replace bookmarks b1 and b2 with resized image Image01.png
            File.Copy(sourceDirectory + "/Test03.docx", destDirectory + "/" + "Test04out.docx");
            image = new Bitmap(@"../../TestDocumentsSource/Image01.png");
            keepBookmark = false;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test04out.docx", true))
            {
                Console.WriteLine("Test04: replacing bookmarks b1 and b2 with different dimensions for Image01.png");
                
                // Bitmap.Width specified in units, where 1 unit = (1/96) inches
                // 1 inch = 914400 emu
                // 1 cm   = 360000 emu

                int imageWidthEmu  = image.Width * ImageElement.emuPerUnit;
                int imageHeightEmu = image.Height * ImageElement.emuPerUnit;

                int imageWidthExtentEmu  = image.Width * ImageElement.emuPerUnit;
                int imageHeightExtentEmu = image.Height * ImageElement.emuPerUnit;

                int[] imageDim1 = new int[] { imageWidthEmu, imageHeightEmu/2, imageWidthExtentEmu, imageHeightExtentEmu};
                int[] imageDim2 = new int[] { imageWidthEmu, imageHeightEmu, imageWidthExtentEmu/2, imageHeightExtentEmu/2 };
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image, ImageFormat.Png, keepBookmark, imageDim1);
                BookmarkReplacer.ReplaceBookmark(doc, "b2", image, ImageFormat.Png, keepBookmark, imageDim2);
            }

            //Test 05: replace bookmarks b1-b5 with Image01-Image05 
            File.Copy(sourceDirectory + "/Test05.docx", destDirectory + "/" + "Test05out.docx");
            image1 = new Bitmap(@"../../TestDocumentsSource/Image01.png");
            image2 = new Bitmap(@"../../TestDocumentsSource/Image02.jpg");
            image3 = new Bitmap(@"../../TestDocumentsSource/Image03.gif");
            image4 = new Bitmap(@"../../TestDocumentsSource/Image04.tiff");
            image5 = new Bitmap(@"../../TestDocumentsSource/Image05.bmp");
            keepBookmark = false;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test05out.docx", true))
            {
                Console.WriteLine("Test05: replacing bookmarks b1-5 with Image01.png, Image02.jpg, etc.");
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image1, ImageFormat.Png, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b2", image2, ImageFormat.Jpeg, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b3", image3, ImageFormat.Gif, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b4", image4, ImageFormat.Tiff, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b5", image5, ImageFormat.Bmp, keepBookmark);
            }

            //Test 06: Place 3 images at b1: Image01.png, Image02.jpg, image03.gif
            File.Copy(sourceDirectory + "/Test06.docx", destDirectory + "/" + "Test06out.docx");
            image1 = new Bitmap(@"../../TestDocumentsSource/Image01.png");
            image2 = new Bitmap(@"../../TestDocumentsSource/Image02.jpg");
            image3 = new Bitmap(@"../../TestDocumentsSource/Image03.gif");
            keepBookmark = true;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test06out.docx", true))
            {
                Console.WriteLine("Test06: placing 3 images at bookmark b1");
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image1, ImageFormat.Png, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image2, ImageFormat.Jpeg, keepBookmark);
                BookmarkReplacer.ReplaceBookmark(doc, "b1", image3, ImageFormat.Gif, keepBookmark);
            }

            //Test 07: place text in center-aligned bookmark A with "abc"
            File.Copy(sourceDirectory + "/" + "Test07.docx", destDirectory + "/" + "Test07out.docx");
            keepBookmark = true;

            using (WordprocessingDocument doc =
                   WordprocessingDocument.Open(destDirectory + "/" + "Test07out.docx", true))
            {
                Console.WriteLine("Test07: placing text abc at bookmark A");
                BookmarkReplacer.ReplaceBookmark(doc, "A", "abc", keepBookmark);
            }
        }
    }
}
